//***********************************************
// Eastek (Daniel Lu) CopyRight 2002'10'03
//
// ClasssName: TTabControlColor;
// Description:   Colorful TabControl
// Properties:
//   1: property RandomColor:
//          yes -> tab colors will different every time you run application.
//          no  -> tab colors will remain same every time.
//   2: property AutoFontColor:
//          yes -> each tab font color will be its inverted tab color
//          no  -> each tab font color will be same as in Font propety.
//
// Functions:
//   F1: Change Tabs Color in real time;
//   F2: Color Tabs Font.
//   F3: Monochrome Tabs Font.
//
//
// Email: daniellu@eastek.com.tw
//************************************************


unit TabControlColor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls;

const
  nColor=256;   //b]w ColorAry ƶq
  BasicColor=120;

type
  TNoYesFlag = (no,yes);
  TFuncGetColor =  function (color:TColor):TColor of object;

  TTabControlColor = class(TTabControl)
  private
    { Private declarations }
    FRandomColor:TNoYesFlag;
    FAutoFontColor:TNoYesFlag;
    GetFontColor: TFuncGetColor;
    procedure SetRandomFlag(flag:TNoYesFlag);
    procedure RefreshColorPtr(flag:TNoYesFlag);
    procedure WMKeyDown(var Message: TMessage); message WM_KEYDOWN;
    procedure WMMouseMove(var Message: TMessage); message WM_MOUSEMOVE;


  protected
    { Protected declarations }
    procedure InitialColorAry;
    procedure DrawTab(TabIndex: Integer; const Rect: TRect; Active: Boolean); override;
    function RemainColor(color:TColor):TColor;
    function InvertColor(color:TColor):TColor;

  public
    { Public declarations }
    TabsColor: Array[0..nColor-1] of TColor;
    FontsColor: Array[0..nColor-1] of TColor;
    constructor Create(AOwner: TComponent); override;
  published
    { Published declarations }
    property RandomColor:TNoYesFlag read FRandomColor write SetRandomFlag default no;
    property AutoFontColor:TNoYesFlag read FAutoFontColor write RefreshColorPtr default No;
  end;

procedure Register;

implementation

procedure TTabControlColor.InitialColorAry;
const
  iR=0;
  iG=1;
  iB=2;
var
  RGBAry:Array[iR..iB] of integer;
  i,j,k,Count,iDivValue,iModValue,isteps:integer;
begin
  //iDivale: nColorCRGB 
  iDivValue:=nColor div 3;
  //iModValue: oѤU
  iModValue:=nColor mod 3;


  //]wRGB򥻶
  for i:=Low(RGBAry) to High(RGBAry) do
    RGBAry[i]:=iDivValue;

  //NѤU A̧Ǥ R -> G -> B
  for i:=Low(RGBAry) to High(RGBAry) do
  if iModValue>0 then
  begin
    inc(RGBAry[i]);
    dec(iModValue);
  end
  else break;

  //iSteps: CRGBb 0..255  Dƭ
  iSteps:=High(BYTE) div RGBAry[iR];
  Count:=0;

  {
    //ZDC
  for i:=1 to RGBAry[iR] do
  for j:=RGBAry[iG] downto 1 do
  for k:=1 to RGBAry[iB] do
  if Count<nColor then
  begin
    TabsColor[Count]:=RGB(Byte(RGBARy[i]*iSteps),Byte(RGBARy[j]*iSteps),Byte(RGBARy[k]*iSteps));
    inc(Count);
  end;


  //üƿC (ptO^
  if yes=RandomColor then Randomize;
  for i:=0 to nColor -1 do
    TabsColor[i]:=RGB(Random(150)+BasicColor, Random(150)+BasicColor, Random(150)+BasicColor);
  }

  //üƿC (jtO^
  if yes=RandomColor then Randomize;
  for i:=0 to nColor-1 do
    TabsColor[i]:=RGB((Random(RGBAry[iR])+1)*iSteps,
                      (Random(RGBAry[iG])+1)*iSteps,
                      (Random(RGBAry[iB])+1)*iSteps);

end;


procedure TTabControlColor.SetRandomFlag(flag:TNoYesFlag);
begin
  FRandomColor:=flag;
  initialColorAry;
end;

procedure TTabControlColor.RefreshColorPtr(flag:TNoYesFlag);
var
  i:integer;
begin
  FAutoFontColor:=flag;
  if No=FAutoFontColor then GetFontColor:=RemainColor
  else GetFontColor:=InvertColor;

  for i:=0 to nColor-1 do
  FontsColor[i]:=GetFontColor(TabsColor[i]);
end;

procedure TTabControlColor.WMKeyDown(var Message: TMessage);
begin
  with Message do
  case Wparam of
    VK_HOME: Message.Msg:=WM_KEYUP; //  Home ɡA^Ĥ@ index p
    VK_F1: begin SetRandomFlag(yes); Refresh; SetRandomFlag(no); Refresh; end;
    VK_F2: begin RefreshColorPtr(yes); Refresh; end;
    VK_F3: begin RefreshColorPtr(no); Refresh; end;
    VK_F12: ;//ShowMessage('Created By Daniel Lu');
  end; 
  inherited;
end;

procedure TTabControlColor.WMMouseMove(var Message: TMessage);
begin   
  inherited;
  //ShowMessage('Mouse Move');
end;

constructor TTabControlColor.Create(AOwner: TComponent);
begin
  InitialColorAry;
  OwnerDraw:=true;
  if No=FAutoFontColor then GetFontColor:=RemainColor
  else GetFontColor:=InvertColor;
  
  inherited Create(AOwner);
end;

procedure TTabControlColor.DrawTab(TabIndex: Integer; const Rect: TRect; Active: Boolean);
var
  tmpFontStyles:TFontStyles;
begin
  inherited DrawTab(TabIndex, Rect, Active);

  with Canvas do
  begin
    pen.color:=TabsColor[tabindex mod nColor];
    brush.color:=pen.Color;//TabsColorAry[tabindex mod nColor];

    //FontsColor[tabindex mod nColor]:=GetFontColor(brush.Color);
    font.Color:=FontsColor[tabindex mod nColor];//GetFontColor(brush.Color);
    if true=Active then
    begin
      tmpFontStyles:=[];
      Include(tmpFontStyles,fsBold);
      //Include(tmpFontStyles,fsItalic);
      Include(tmpFontStyles,fsUnderline);
      font.Style:=tmpFontStyles;
    end;

    TextRect(Rect,Rect.Left+5,5,Tabs[TabIndex]);


    {//[J Active ɵe
    if true=Active then
    begin
      pen.Color:=clBlack;
      brush.Color:=clWhite;//font.Color;
      //Rectangle(Rect.left,Rect.top,Rect.right,Rect.Top+(Rect.Bottom-Rect.top) div 2);
      Ellipse(Rect.right-18,Rect.top+3,Rect.right-3,Rect.top+18);
    end; }
    
  end;
  
end;

function TTabControlColor.RemainColor(color:TColor):TColor;
begin
  result:=Canvas.font.color;
end;


function TTabControlColor.InvertColor(color:TColor):TColor;
begin
  result:=RGB(not (color and $000000FF),
              not (color and $0000FF00 shr 8),
              not (color and $00FF0000 shr 16));
end;



procedure Register;
begin
  RegisterComponents('Win32', [TTabControlColor]);
end;


end.
